<?php
/**
 * 签名助手 2017/11/19
 *
 * Class sms
 */
class Sms {
    var $client; // 客户端
    var $storage_sms_token; // 微信小程序需要使用storage储存token
    var $accessKeyId; // 请参阅 https://ak-console.aliyun.com/ 取得您的AK信息
    var $accessKeySecret; // 为空时默认跟主图在一个目录
    var $SignName; // 短信签名，应严格按"签名名称"填写
    var $security; // 是否启用https
    
    /**
     * +----------------------------------------------------------
     * 构造函数
     * +----------------------------------------------------------
     */
    function __construct($client = 'web', $storage_sms_token = '', $accessKeyId = '', $accessKeySecret = '', $SignName = '', $security = false) {
        $this->client = $client;
        $this->storage_sms_token = $storage_sms_token;
        $this->accessKeyId = $accessKeyId ? $accessKeyId : $GLOBALS['_PARAM']['sms_accessKeyId'];
        $this->accessKeySecret = $accessKeySecret ? $accessKeySecret : $GLOBALS['_PARAM']['sms_accessKeySecret'];
        $this->SignName = $SignName ? $SignName : $GLOBALS['_PARAM']['sms_SignName'];
        $this->security = $security;
    }
 
    // 构造函数.兼容PHP4
    function Sms($client = 'web', $storage_sms_token = '', $accessKeyId = '', $accessKeySecret = '', $SignName = '', $security = false) {
        $this->__construct($client, $storage_sms_token, $accessKeyId, $accessKeySecret, $SignName, $security);
    }
    
    /**
     * +----------------------------------------------------------
     * 发送短信
     * +----------------------------------------------------------
     */
    function send_sms($phone_number, $TemplateCode, $TemplateParam) {
        $params = array ();

        // *** 需用户填写部分 ***
        // 必填：是否启用https
        $security = $this->security;

        // 必填: 请参阅 https://ak-console.aliyun.com/ 取得您的AK信息
        $accessKeyId = $this->accessKeyId;
        $accessKeySecret = $this->accessKeySecret;

        // 必填: 短信接收号码
        $params["PhoneNumbers"] = $phone_number;

        // 必填: 短信签名，应严格按"签名名称"填写，请参考: https://dysms.console.aliyun.com/dysms.htm#/develop/sign
        $params["SignName"] = $this->SignName;

        // 必填: 短信模板Code，应严格按"模板CODE"填写, 请参考: https://dysms.console.aliyun.com/dysms.htm#/develop/template
        $params["TemplateCode"] = $TemplateCode;

        // 必填: 下发的参数, 数组形式
        $params['TemplateParam'] = $TemplateParam;

        // 可选: 设置发送短信流水号
        $params['OutId'] = "12345";

        // 可选: 上行短信扩展码, 扩展码字段控制在7位或以下，无特殊需求用户请忽略此字段
        $params['SmsUpExtendCode'] = "1234567";


        // *** 需用户填写部分结束, 以下代码若无必要无需更改 ***
        if(!empty($params["TemplateParam"]) && is_array($params["TemplateParam"])) {
            $params["TemplateParam"] = json_encode($params["TemplateParam"], JSON_UNESCAPED_UNICODE);
        }

        // 此处可能会抛出异常，注意catch
        $content = $this->request(
            $accessKeyId,
            $accessKeySecret,
            "dysmsapi.aliyuncs.com",
            array_merge($params, array(
                "RegionId" => "cn-hangzhou",
                "Action" => "SendSms",
                "Version" => "2017-05-25",
            )),
            $security
        );

        $code = $content->Code;
        if ($code == 'OK') {
            return 'success';
        } elseif ($code == 'isv.MOBILE_NUMBER_ILLEGAL') {
            return '手机号输入有误，请重新输入';
        } elseif ($code == 'isv.TEMPLATE_MISSING_PARAMETERS') {
            return '模板内容中的变量未全部赋值';
        } elseif ($code == 'isv.SMS_TEMPLATE_ILLEGAL') {
            return '短信模板不存在，或未经审核通过';
        } elseif ($code == 'isv.SMS_SIGNATURE_ILLEGAL	短信签名不合法') {
            return '签名不存在，或未经审核通过';
        } elseif ($code == 'isv.OUT_OF_SERVICE') {
            return '短信账户余额不足';
        } elseif ($code == 'isv.BUSINESS_LIMIT_CONTROL') {
            return '短信发送频率超限';
        } elseif ($code == 'isv.AMOUNT_NOT_ENOUGH') {
            return '阿里云账户余额不足';
        } else {
            return $code;
        }
    }
 
    /**
     * +----------------------------------------------------------
     * 设置令牌
     * +----------------------------------------------------------
     */
    function sms_token_set() {
        $sms_token = strtoupper($GLOBALS['dou']->create_rand_string('letter.number', 5));
        
        $temp_id = $this->sms_temp_id();
      
        if ($this->client == 'miniprogram') {
            $data['storage_sms_token'] = md5($sms_token . DOU_SHELL);
            $data['sms_token'] = $sms_token;
            
            return $data;
        } else {
            $_SESSION[$temp_id] = md5($sms_token . DOU_SHELL);
            
            return $sms_token;
        }
    }
 

    /**
     * +----------------------------------------------------------
     * 验证令牌（如果是在后台操作，将采用每次登录更新令牌）
     * +----------------------------------------------------------
     */
    function sms_token_check($sms_token) {
        if ($this->client == 'miniprogram') {
            if ($this->storage_sms_token && $sms_token == $this->storage_sms_token) {
                return 'ok'; // 返回令牌验证成功标记
            } else {
                return 'wrong';
            }
        } else {
            $temp_id = $this->sms_temp_id();
            if (isset($_SESSION[$temp_id]) && $sms_token == $_SESSION[$temp_id]) {
                return 'ok';
            } else {
                return 'wrong';
            }
        }
    }
 
    /**
     * +----------------------------------------------------------
     * 设置短信令牌零时ID
     * +----------------------------------------------------------
     */
    function sms_temp_id() {
        $temp_id = substr(md5('sms' . DOU_SHELL) , 0 , 16);
        return $temp_id;
    }
 
    /**
     * +----------------------------------------------------------
     * 登录验证有效期为5分钟(300秒)
     * +----------------------------------------------------------
     * $ontime 发送验证码的时间
     * $timeout 超时时间
     * +----------------------------------------------------------
     */
    function sms_ontime($ontime = '', $timeout = 300) {
        $cur_time = time(); // 当前时间
        if ($cur_time - $ontime < $timeout) { // 当前时间减去发送时间的值与超时时间比较
            return true;
        }
    }
 
    /**
     * +----------------------------------------------------------
     * 短信验证码
     * +----------------------------------------------------------
     */
    function captcha($sms_token, $telphone) {
        if ($this->sms_token_check($sms_token)) {
            $telphone = $GLOBALS['check']->is_telphone($telphone) ? $telphone : '';

            if (!empty($telphone)) {
                $TemplateCode['code'] = $GLOBALS['dou']->create_rand_string('number', 4); // 随机验证码
                $msg = $this->send_sms($telphone, $GLOBALS['_PARAM']['sms_TemplateCode'], $TemplateCode); // 发送短信并返回信息给AJAX
                
                if ($msg == 'success') {
                    $_SESSION[DOU_ID]['sms']['telphone'] = $telphone; // 缓存发送验证码的手机号
                    $_SESSION[DOU_ID]['sms']['ontime'] = time();
                    $_SESSION[DOU_ID]['sms']['code'] = md5($TemplateCode['code'] . $telphone . DOU_SHELL);
                    $_SESSION[DOU_ID]['sms']['number'] = $TemplateCode['code'];
                 
                    if ($this->client == 'miniprogram')
                        $response['sms'] = $_SESSION[DOU_ID]['sms'];
                }
                $response['msg'] = $msg;
            } else {
                $response['msg'] = $GLOBALS['_LANG']['user_telphone_cue'];
            }
        } else {
            $response['msg'] = $GLOBALS['_LANG']['illegal'];
        }
     
        return $response;
    }

    /**
     * 生成签名并发起请求
     *
     * @param $accessKeyId string AccessKeyId (https://ak-console.aliyun.com/)
     * @param $accessKeySecret string AccessKeySecret
     * @param $domain string API接口所在域名
     * @param $params array API具体参数
     * @param $security boolean 使用https
     * @param $method boolean 使用GET或POST方法请求，VPC仅支持POST
     * @return bool|\stdClass 返回API接口调用结果，当发生错误时返回false
     */
    public function request($accessKeyId, $accessKeySecret, $domain, $params, $security=false, $method='POST') {
        $apiParams = array_merge(array (
            "SignatureMethod" => "HMAC-SHA1",
            "SignatureNonce" => uniqid(mt_rand(0,0xffff), true),
            "SignatureVersion" => "1.0",
            "AccessKeyId" => $accessKeyId,
            "Timestamp" => gmdate("Y-m-d\TH:i:s\Z"),
            "Format" => "JSON",
        ), $params);
        ksort($apiParams);

        $sortedQueryStringTmp = "";
        foreach ($apiParams as $key => $value) {
            $sortedQueryStringTmp .= "&" . $this->encode($key) . "=" . $this->encode($value);
        }

        $stringToSign = "{$method}&%2F&" . $this->encode(substr($sortedQueryStringTmp, 1));

        $sign = base64_encode(hash_hmac("sha1", $stringToSign, $accessKeySecret . "&",true));

        $signature = $this->encode($sign);

        $url = ($security ? 'https' : 'http')."://{$domain}/";

        try {
            $content = $this->fetchContent($url, $method, "Signature={$signature}{$sortedQueryStringTmp}");
            return json_decode($content);
        } catch( Exception $e) { // 如果有抛出错误信息
            return false;
        }
    }

    private function encode($str)
    {
        $res = urlencode($str);
        $res = preg_replace("/\+/", "%20", $res);
        $res = preg_replace("/\*/", "%2A", $res);
        $res = preg_replace("/%7E/", "~", $res);
        return $res;
    }

    private function fetchContent($url, $method, $body) {
        $ch = curl_init();

        if($method == 'POST') {
            curl_setopt($ch, CURLOPT_POST, 1);//post提交方式
            curl_setopt($ch, CURLOPT_POSTFIELDS, $body);
        } else {
            $url .= '?'.$body;
        }

        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_TIMEOUT, 5);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "x-sdk-client" => "php/2.0.0"
        ));

        if(substr($url, 0,5) == 'https') {
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        }

        $rtn = curl_exec($ch);

        if($rtn === false) {
            // 大多由设置等原因引起，一般无法保障后续逻辑正常执行，
            // 所以这里触发的是E_USER_ERROR，会终止脚本执行，无法被try...catch捕获，需要用户排查环境、网络等故障
            // trigger_error("[CURL_" . curl_errno($ch) . "]: " . curl_error($ch), E_USER_ERROR);
        }
        curl_close($ch);

        return $rtn;
    }
}